package data.hsqldb;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import data.Abteilung;
import data.Angestellter;
import data.Chef;
import data.Person;

public class AngestellterMapper extends PersonMapper {
	public final static String insertString = "insert into person (dtype,vorname,nachname,abteilung_id,chef_id) values (?,?,?,?,?)";
	public final static String updateString = "update person set vorname=?, nachname=?, abteilung_id=?, chef_id=? where id=?";

	public final static int DTYPE = 1;
	public AngestellterMapper(DataProviderHSQL data) {
		super(data);
	}
	
	@Override
	public Angestellter load(Integer id) {
		return (Angestellter) super.load(id);
	}

	@Override
	public void insert(Object o) {
		if (! (o instanceof Angestellter)) throw new RuntimeException();
		Angestellter a = (Angestellter) o;
		Connection connection = data.getConnection();
		if (a.getId()==null) {
			try {
				PreparedStatement st = connection.prepareStatement(insertString);
				st.setInt(1, this.DTYPE);
				st.setString(2, a.getVorname());
				st.setString(3, a.getNachname());
				if (a.getAbteilung()==null) {
					st.setNull(4, java.sql.Types.INTEGER);
				} else {
					st.setInt(4, a.getAbteilung().getId());					
				}
				if (a.getChef()==null) {
					st.setNull(5, java.sql.Types.INTEGER);
				} else {
					st.setInt(5, a.getChef().getId());
				}
				st.executeUpdate();
				st = connection.prepareStatement("call IDENTITY()");
				ResultSet res = st.executeQuery();
				res.next();
				connection.commit();
				Integer id = res.getInt(1);
				identity_map.put(id, a);
				a.setId(id);
			} catch (SQLException exc) {
				exc.printStackTrace();
			}
		} else {
			update(a);
		}
	}

	@Override
	public void update(Object o) {
		if (! (o instanceof Angestellter)) throw new RuntimeException();
		Angestellter a = (Angestellter) o;
		Connection connection = data.getConnection();
		if (a.getId()!=null) {
			try {
			PreparedStatement st = connection.prepareStatement(updateString);
			st.setString(1, a.getVorname());
			st.setString(2, a.getNachname());
			if (a.getAbteilung()==null) {
				st.setNull(3, java.sql.Types.INTEGER);
			} else {
				st.setInt(3, a.getAbteilung().getId());
			}
			if (a.getChef()==null) {
				st.setNull(4, java.sql.Types.INTEGER);
			} else {
				st.setInt(4, a.getChef().getId());
			}
			st.setInt(5, a.getId());
			st.executeUpdate();
			connection.commit();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		} else {
			insert(a);
		}
	}

	@Override
	protected Angestellter load(ResultSet res) throws SQLException {
		Integer id = res.getInt("id");
		Angestellter a = (Angestellter) identity_map.get(id);
		if (a!=null) return a;
		Integer dtype = res.getInt("dtype");
		if (dtype!=this.DTYPE) throw new RuntimeException();
		a = new Angestellter();
		a.setId(id);
		a.setVorname(res.getString("vorname"));
		a.setNachname(res.getString("nachname"));
		int abteilung_id = res.getInt("abteilung_id");
		if (abteilung_id==0) {
			a.setAbteilung(null);
		} else {
			a.setAbteilung(data.load(Abteilung.class, abteilung_id));
		}
		int chef_id = res.getInt("chef_id");
		if (chef_id==0) {
			a.setChef(null);
		} else {
			a.setChef(data.load(Chef.class, chef_id));
		}
		identity_map.put(a.getId(), a);
		return a;
	}
}
