package data.hsqldb;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import data.Abteilung;
import data.Angestellter;
import data.Chef;
import data.Person;

public class ChefMapper extends PersonMapper {
	public final static String insertString = "insert into person (dtype,vorname,nachname,abteilung_id,budget) values (?,?,?,?,?)";
	public final static String updateString = "update person set vorname=?, nachname=?, abteilung_id=?, budget=? where id=?";

	public final static int DTYPE = 2;
	public ChefMapper(DataProviderHSQL data) {
		super(data);
	}

	@Override
	public void insert(Object o) {
		if (! (o instanceof Chef)) throw new RuntimeException();
		Chef c = (Chef) o;
		Connection connection = data.getConnection();
		if (c.getId()==null) {
			try {
				PreparedStatement st = connection.prepareStatement(insertString);
				st.setInt(1, this.DTYPE);
				st.setString(2, c.getVorname());
				st.setString(3, c.getNachname());
				if (c.getAbteilung()==null) {
					st.setNull(4, java.sql.Types.INTEGER);
				} else {
					st.setInt(4, c.getAbteilung().getId());					
				}
				st.setInt(5, c.getBudget());
				st.executeUpdate();
				st = connection.prepareStatement("call IDENTITY()");
				ResultSet res = st.executeQuery();
				res.next();
				connection.commit();
				Integer id = res.getInt(1);
				identity_map.put(id, c);
				c.setId(id);
			} catch (SQLException exc) {
				exc.printStackTrace();
			}
		} else {
			update(c);
		}
	}

	@Override
	public void update(Object o) {
		if (! (o instanceof Chef)) throw new RuntimeException();
		Chef c = (Chef) o;
		Connection connection = data.getConnection();
		if (c.getId()!=null) {
			try {
			PreparedStatement st = connection.prepareStatement(updateString);
			st.setString(1, c.getVorname());
			st.setString(2, c.getNachname());
			if (c.getAbteilung()==null) {
				st.setNull(3, java.sql.Types.INTEGER);
			} else {
				st.setInt(3, c.getAbteilung().getId());
			}
			st.setInt(4, c.getBudget());
			st.setInt(5, c.getId());
			st.executeUpdate();
			connection.commit();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		} else {
			insert(c);
		}
	}

	@Override
	public Chef load(Integer id) {
		return (Chef) super.load(id);
	}

	@Override
	protected Chef load(ResultSet res) throws SQLException {
		Integer id = res.getInt("id");
		Chef c = (Chef) identity_map.get(id);
		if (c!=null) return c;
		Integer dtype = res.getInt("dtype");
		if (dtype!=this.DTYPE) throw new RuntimeException();
		c = new Chef();
		c.setId(id);
		c.setVorname(res.getString("vorname"));
		c.setNachname(res.getString("nachname"));
		int abteilung_id = res.getInt("abteilung_id");
		if (abteilung_id==0) {
			c.setAbteilung(null);
		} else {
			c.setAbteilung(data.load(Abteilung.class, abteilung_id));
		}
		c.setBudget(res.getInt("budget"));
		identity_map.put(c.getId(), c);
		return c;
	}

}
