package data.hsqldb;

import java.sql.*;
import java.util.*;

import javax.management.RuntimeErrorException;

import data.*;

public class PersonMapper extends AbstractMapper<Person> {
	public final static String selectString = "select id,dtype,vorname,nachname,abteilung_id,chef_id,budget from person";
	public final static String deleteString = "delete from person where id=?";
	
	public PersonMapper(DataProviderHSQL data) {
		super(data);
	}

	public Mapper<? extends Person> getMapper(Integer dtype) {
		switch (dtype) {
		case AngestellterMapper.DTYPE:
			return data.getMapper(Angestellter.class);
		case ChefMapper.DTYPE:
			return data.getMapper(Chef.class);
		default:
			return null;
		}
	}
	
	@Override
	public Person load(Integer id) {
		Person p = (Person) identity_map.get(id);
		if (p!=null) return p;
		
		Connection c = data.getConnection();
		
		try {
			PreparedStatement st = c.prepareStatement(selectString+" where id=?");
			st.setInt(1, id);
			List<Person> l = (List<Person>) load(st, new ArrayList<Person>());
			if (l.isEmpty()) return null;
			p = l.get(0);
			identity_map.put(p.getId(), p);
			return p;
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}
	}

	@Override
	public void insert(Object o) {
		throw new RuntimeException("not supported");
	}

	@Override
	public void update(Object o) {
		throw new RuntimeException("not supported");
	}

	@Override
	public void delete(Object o) {
		if (! (o instanceof Person)) throw new RuntimeException();
		Person p = (Person) o;
		Connection connection = data.getConnection();
		if (p.getId()==null) throw new RuntimeException("p not persistent");
		try {
			PreparedStatement st = connection.prepareStatement(deleteString);
			st.setInt(1, p.getId());
			st.executeUpdate();
			identity_map.remove(p.getId());
			p.setId(null);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	@Override
	protected Person load(ResultSet res) throws SQLException {
		Integer id = res.getInt("id");
		Person p = (Person) identity_map.get(id);
		if (p!=null) return p;
		Integer dtype = res.getInt("dtype");
		PersonMapper mapper = (PersonMapper) getMapper(dtype);
		return mapper.load(res);
	}
}
